<?php
// +----------------------------------------------------------------------+
// | Copyright 2005-2009 the Seasar Foundation and the Others.            |
// +----------------------------------------------------------------------+
// | Licensed under the Apache License, Version 2.0 (the "License");      |
// | you may not use this file except in compliance with the License.     |
// | You may obtain a copy of the License at                              |
// |                                                                      |
// |     http://www.apache.org/licenses/LICENSE-2.0                       |
// |                                                                      |
// | Unless required by applicable law or agreed to in writing, software  |
// | distributed under the License is distributed on an "AS IS" BASIS,    |
// | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,                        |
// | either express or implied. See the License for the specific language |
// | governing permissions and limitations under the License.             |
// +----------------------------------------------------------------------+
/**
 * @copyright 2005-2009 the Seasar Foundation and the Others.
 * @license   http://www.apache.org/licenses/LICENSE-2.0
 * @link      http://s2container.php5.seasar.org/
 * @version   SVN: $Id:$
 * @since     Class available since Release 2.0.0
 * @package   seasar.container.deployer
 * @author    klove
 */
namespace seasar\container\deployer;
class SingletonComponentDeployerTest extends \PHPUnit_Framework_TestCase {

    public function testAssembleWithoutArgs() {
        $container = new \seasar\container\impl\S2ContainerImpl();
        $componentDef = new \seasar\container\impl\ComponentDefImpl(__NAMESPACE__ . '\A_SingletonComponentDeployerTest');
        $deployer = new SingletonComponentDeployer($componentDef);
        $this->assertTrue($deployer->deploy() instanceof \seasar\container\deployer\A_SingletonComponentDeployerTest);
    }

    public function testSingleton() {
        $container = new \seasar\container\impl\S2ContainerImpl();
        $componentDef = new \seasar\container\impl\ComponentDefImpl(__NAMESPACE__ . '\A_SingletonComponentDeployerTest');
        $deployer = new SingletonComponentDeployer($componentDef);
        $this->assertTrue($deployer->deploy() === $deployer->deploy());
    }

    public function testCyclicReference1Step() {
        $container = new \seasar\container\impl\S2ContainerImpl();
        $componentDef = new \seasar\container\impl\ComponentDefImpl(__NAMESPACE__ . '\B_SingletonComponentDeployerTest');
        $container->register($componentDef);
        $deployer = new SingletonComponentDeployer($componentDef);
        try {
            $deployer->deploy();
            $this->fail();
        } catch(\seasar\container\exception\CyclicReferenceRuntimeException $e) {
            print $e->getMessage() .PHP_EOL;
        } catch(Exception $e) {
            print $e->getMessage() .PHP_EOL;
            $this->fail();
        }
    }

    public function testCyclicReference2Step() {
        $container = new \seasar\container\impl\S2ContainerImpl();
        $componentDef = new \seasar\container\impl\ComponentDefImpl(__NAMESPACE__ . '\C_SingletonComponentDeployerTest');
        $container->register($componentDef);
        $componentDef = new \seasar\container\impl\ComponentDefImpl(__NAMESPACE__ . '\D_SingletonComponentDeployerTest');
        $container->register($componentDef);
        $deployer = new SingletonComponentDeployer($componentDef);
        try {
            $deployer->deploy();
            $this->fail();
        } catch(\seasar\container\exception\CyclicReferenceRuntimeException $e) {
            print $e->getMessage() .PHP_EOL;
        } catch(Exception $e) {
            print $e->getMessage() .PHP_EOL;
            $this->fail();
        }
    }

    public function setUp(){
        print PHP_EOL . __CLASS__ . '->' . $this->getName() . '()' . PHP_EOL;
    }

    public function tearDown() {
    }
}

class A_SingletonComponentDeployerTest {}

class B_SingletonComponentDeployerTest {
    public function __construct(B_SingletonComponentDeployerTest $b) {
    }
}

class C_SingletonComponentDeployerTest {
    public function __construct(D_SingletonComponentDeployerTest $b) {
    }
}

class D_SingletonComponentDeployerTest {
    public function __construct(C_SingletonComponentDeployerTest $b) {
    }
}
